/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <tiuser.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>

#define CMDSZ	256

void timeout(int);
extern int initclient(char *);
extern int connect(int, char *, uint_t);

void
rcmd(int argc, char *argv[], char *transport, char *addr,
	uint_t alen)
{
	int fd, i, n, flags;
	struct sigaction sa;
	char buf[CMDSZ];

	if (argc < 3) {
		fprintf(stderr,
		  "usage: rcmd host command [options]\n");
		exit(1);
	}

	/*
	 * Create a client-style transport endpoint.
	 */
	if ((fd = initclient(transport)) < 0)
		exit(1);

	/*
	 * Create a connection to the server.
	 */
	if (connect(fd, addr, alen) < 0)
		exit(1);

	/*
	 * Copy the command and its arguments to buf.  The
	 * command string is NULL-terminated and buf[0]
	 * contains the total number of bytes to be sent.
	 */
	strcpy(&buf[1], argv[2]);
	for (i = 3; i < argc; i++) {
		strcat(&buf[1], " ");	/* add a space */
		strcat(&buf[1], argv[i]);
	}
	i = strlen(&buf[1]);
	buf[i+1] = '\0';
	buf[0] = i + 2;

	/*
	 * Set an alarm so we don't block forever.
	 */
	sa.sa_handler = timeout;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);
	sigaction(SIGALRM, &sa, NULL);
	alarm(60);

	/*
	 * Send the command string to the server.
	 */
	if ((n = t_snd(fd, buf, i, 0)) != i) {
		if (n < 0)
			t_error("rcmd: Cannot send request");
		else
			fprintf(stderr, "rcmd: Partial transmission");
		exit(1);
	}

	/*
	 * Receive response messages until we encounter an
	 * error or receive the end of the messages (a single
	 * byte containing 0).
	 */
	while ((n = t_rcv(fd, buf, CMDSZ, &flags)) > 0) {
		if (buf[n-1] == '\0') {
			n--;
			break;
		}
		/*
		 * Write the response to the standard output.
		 */
		write(1, buf, n);
	}

	/*
	 * The final byte might have been appended to the
	 * preceding message, so we might have more to write.
	 */
	if (n > 0) {
		write(1, buf, n);
	} else if (n < 0) {
		t_error("rcmd: Cannot receive data");
		exit(1);
	}
	exit(0);
}

void
timeout(int sig)
{
	fprintf(stderr, "rcmd: Command timed out\n");
	exit(1);
}
